/*
 * $Header: /home/cvs/jakarta-slide/src/share/org/apache/slide/common/EmbeddedDomain.java,v 1.4 2001/09/25 07:34:45 cmlenz Exp $
 * $Revision: 1.4 $
 * $Date: 2001/09/25 07:34:45 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.slide.common;

import java.util.Hashtable;
import java.util.Enumeration;
import java.io.IOException;
import org.apache.slide.lock.*;
import org.apache.slide.util.conf.*;
import org.apache.slide.util.logger.Logger;
import org.apache.slide.util.logger.SimpleLogger;

/**
 * Alternate domain designed to ease embedding.
 *
 * @author Remy Maucherat
 * @version $Revision: 1.4 $
 */
public class EmbeddedDomain {
    
    
    // ------------------------------------------------------------ Constructor
    
    
    /**
     * Default constructor.
     */
    public EmbeddedDomain() {

        // Compatibility with the static domain
        if (!Domain.isInitialized())
            Domain.setDomain(this);

    }
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Namespaces hashtable.
     */
    private Hashtable namespaces = new Hashtable();
    
    
    /**
     * Default namespace.
     */
    private String defaultNamespace;
    
    
    // --------------------------------------------------------- Public Methods
    
    
    /**
     * Return the default namespace of this domain.
     *
     * @return the name of the default namespace
     */
    public String getDefaultNamespace() {
        return defaultNamespace;
    }
    
    
    /**
     * Set the default namespace of this domain.
     * 
     * @param defaultNamespace New default namespace name
     */
    public void setDefaultNamespace(String defaultNamespace) {
        this.defaultNamespace = defaultNamespace;
    }
    
    
    /**
     * Set the logger to be used by Slide.
     *
     * @param logger Logger the domain will use
     * @deprecated Use the namespace loggers instead
     */
    public void setLogger(Logger logger) {
        Domain.setLogger(logger);
    }
    
    
    /**
     * Get the Domain logger.
     *
     * @return The domain logger
     * @deprecated Use the namespace loggers instead
     */
    public Logger getLogger() {
        return Domain.getLogger();
    }
    
    
    /**
     * Access a Namespace.
     *
     * @param namespaceName Name of the namespace on which access is requested
     * @return NamespaceAccessToken Access token to the namespace
     */
    public NamespaceAccessToken getNamespaceToken(String namespaceName) {
        Namespace namespace = (Namespace) namespaces.get(namespaceName);
        if (namespace == null)
            return null;
        else
            return new NamespaceAccessTokenImpl(namespace);
    }
    
    
    /**
     * Enumerate namespace names.
     */
    public Enumeration enumerateNamespaces() {
        return (namespaces.keys());
    }
    
    
    /**
     * Add a namespace to this domain.
     * 
     * @param name Namespace name
     * @param logger Namespace logger
     * @param definition Input stream to the namepace definition
     * @param configuration Input stream to the namespace configuration
     * @param baseData Input stream to the anmespace base data
     */
    public Namespace addNamespace(String name, Logger logger,
                                  Configuration definition,
                                  Configuration configuration,
                                  Configuration baseData) {

        // FIXME: Check parameters

        Namespace namespace = new Namespace();
        namespace.setName(name);
        namespace.setLogger(logger);

        try {
            namespace.loadDefinition(definition);
            namespace.loadParameters(configuration);
            namespace.loadBaseData(baseData);
            namespace.loadConfiguration(configuration);
        } catch (SlideException e) {
            e.printStackTrace();
            return null;
        } catch (ConfigurationException e) {
            e.printStackTrace();
            return null;
        }

        namespaces.put(name, namespace);

        return namespace;

    }
    
    
    /**
     * Clsose a namespace.
     *
     * @param name Name of the namespace
     */
    public Namespace removeNamespace(String name) {

        // FIXME: Check parameters

        try {
            Namespace namespace = (Namespace) namespaces.get(name);
            namespace.disconnectServices();
        } catch(Exception e) {
        }
        return (Namespace) namespaces.remove(name);

    }
    
    
    /**
     * Get a namespace.
     *
     * @param name Name of the namespace
     * @return Namespace
     */
    public Namespace getNamespace(String name) {
        return (Namespace) namespaces.get(name);
    }
    
    
    /**
     * Start domain (doesn't do anything yet).
     */
    public void start()
        throws Exception {
        
    }
    
    
    /**
     * Stop domain.
     */
    public void stop()
        throws Exception {
        
        Enumeration active = namespaces.elements();
        while (active.hasMoreElements()) {
            ((Namespace) active.nextElement()).disconnectServices();
        }
        
    }
    
    
}
