/*
 * $Header: /home/cvs/jakarta-slide/src/share/org/apache/slide/security/SecurityImpl.java,v 1.27 2001/09/12 13:50:32 juergen Exp $
 * $Revision: 1.27 $
 * $Date: 2001/09/12 13:50:32 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.slide.security;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import org.apache.slide.common.*;
import org.apache.slide.structure.*;
import org.apache.slide.authenticate.CredentialsToken;
import org.apache.slide.util.Configuration;

/**
 * Security helper.
 *
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.27 $
 */
public final class SecurityImpl implements Security {
    
    
    // ----------------------------------------------------------- Constructors
    
    
    /**
     * Constructor.
     *
     * @param namespace Namespace
     * @param namespaceConfig Namespace configuration
     */
    public SecurityImpl(Namespace namespace, NamespaceConfig namespaceConfig) {
        this.namespace = namespace;
        this.namespaceConfig = namespaceConfig;
        this.rolesCache = new Hashtable();
    }
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Namespace.
     */
    private Namespace namespace;
    
    
    /**
     * Namespace configuration.
     */
    private NamespaceConfig namespaceConfig;
    
    
    /**
     * Roles cache.
     * Role name -> Role interface.
     */
    private Hashtable rolesCache;
    
    
    // ------------------------------------------------------- Security Methods
    
    
    /**
     * Set a new set of permissions on an object.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    public void setPermissions(SlideToken token, String object,
                               Enumeration permissions)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException {
        
        Uri objectUri = namespace.getUri(token, object);
        ObjectNode objectNode = objectUri.getStore().retrieveObject(objectUri);
        
        checkCredentials(token, objectNode,
                         namespaceConfig.getGrantPermissionAction());
        checkCredentials(token, objectNode,
                         namespaceConfig.getRevokePermissionAction());
        
        objectUri.getStore().revokePermissions(objectUri);
        
        while (permissions.hasMoreElements()) {
            NodePermission permission =
                (NodePermission) permissions.nextElement();
            objectUri.getStore().grantPermission(objectUri, permission);
        }
        
    }
    
    
    /**
     * Grants a new permission.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @param activity Activity which the actor can perform
     * @param collection Collection on which the actor can perform
     * the specified action
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    public void grantPermission(SlideToken token, ObjectNode object,
                                SubjectNode subject, ActionNode action)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException {
        grantPermission(token, object, subject, action, true);
    }
    
    
    /**
     * Grants a new permission.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @param subject Subject who can perform the action
     * @param action Action which can be performed
     * @param inheritable Create an inheritable permission
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    public void grantPermission(SlideToken token, ObjectNode object,
                                SubjectNode subject, ActionNode action,
                                boolean inheritable)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException {
        NodePermission permission = new NodePermission(object, subject, action,
                                                       inheritable);
        grantPermission(token, permission);
    }
    
    
    /**
     * Grants a new permission.
     *
     * @param token Credentials token
     * @param permission New permission
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    public void grantPermission(SlideToken token,
                                NodePermission permission)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException {
        Uri objectUri = namespace.getUri(token, permission.getObjectUri());
        ObjectNode object = objectUri.getStore()
            .retrieveObject(objectUri);
        
        // Checking if the permission is already present
        Enumeration permissions = enumeratePermissions(token, object);
        boolean alreadyPresent = false;
        while (permissions.hasMoreElements() && !alreadyPresent) {
            if (permission.equals(permissions.nextElement())) {
                alreadyPresent = true;
            }
        }
        
        if (!alreadyPresent) {
            checkCredentials(token, object,
                             namespaceConfig.getGrantPermissionAction());
            objectUri.getStore().grantPermission(objectUri, permission);
        }
    }
    
    
    /**
     * Deny a new permission.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @param activity Activity which the actor can perform
     * @param collection Collection on which the actor can perform
     * the specified action
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    public void denyPermission(SlideToken token, ObjectNode object,
                               SubjectNode subject, ActionNode action)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException {
        denyPermission(token, object, subject, action, true);
    }
    
    
    /**
     * Deny a new permission.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @param subject Subject who can perform the action
     * @param action Action which can be performed
     * @param inheritable Create an inheritable permission
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    public void denyPermission(SlideToken token, ObjectNode object,
                               SubjectNode subject, ActionNode action,
                               boolean inheritable)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException {
        NodePermission permission = new NodePermission(object, subject, action,
                                                       inheritable, true);
        denyPermission(token, permission);
    }
    
    
    /**
     * Deny a new permission.
     *
     * @param token Credentials token
     * @param permission New permission
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    public void denyPermission(SlideToken token,
                                NodePermission permission)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException {
        // Make sure the permission we're about to set is indeed a negative
        // permission
        if (!permission.isNegative())
            permission.setNegative(true);
        grantPermission(token, permission);
    }
     
    
    /**
     * Revokes a permission.
     *
     * @param token Credentials token
     * @param object Object on which permission is revoked
     * @param subject Subject who can perform the action
     * @param action Action which can be performed
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    public void revokePermission(SlideToken token, ObjectNode object,
                                 SubjectNode subject, ActionNode action)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException {
        //Domain.info("Revoke permission on " + object.getUri());
        checkCredentials(token, object, namespaceConfig
                         .getRevokePermissionAction());
        NodePermission permission = new NodePermission(object, subject,
                                                       action);
        Uri objectUri = namespace.getUri(token, object.getUri());
        objectUri.getStore()
            .revokePermission(objectUri, permission);
    }
    
    
    
    /**
     * Revokes a permission.
     *
     * @param token Credentials token
     * @param permission Permission to be removed
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    public void revokePermission(SlideToken token, NodePermission permission)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException {
            
        Uri objectUri = namespace.getUri(token, permission.getObjectUri());
        ObjectNode object = objectUri.getStore().retrieveObject(objectUri);
            
        checkCredentials(token, object,
                         namespaceConfig.getRevokePermissionAction());
        objectUri.getStore().revokePermission(objectUri, permission);
    }
    
    
    /**
     * Check if the credentials given grants permission to perform
     * the specified action on the specified subject.
     *
     * @param token Credentials token
     * @param object Object on which the action is performed
     * @param action Action performed
     * @exception ServiceAccessException DataSource access error
     * @exception AccessDeniedException The credentials does not grant
     * the permission to perform the specified action
     */
    public void checkCredentials(SlideToken token, ObjectNode object,
                                 ActionNode action)
        throws ServiceAccessException, AccessDeniedException {
        token = new SlideTokenWrapper(token, false);  // read only access
        try {
            if (Configuration.useIntegratedSecurity()) {
                Uri objectUri = namespace.getUri(token, object.getUri());
                ObjectNode realObject = objectUri.getStore()
                    .retrieveObject(objectUri);
                SubjectNode subject = (SubjectNode) getPrincipal(token);
                checkPermission(realObject, subject, action);
            }
        } catch (ObjectNotFoundException e) {
            throw new AccessDeniedException
                (object.getUri(),
                 token.getCredentialsToken().getPublicCredentials(),
                 action.getUri());
        }
    }
    
    
    /**
     * Check whether or not an actor can perform the specified activity
     * on a collection.
     *
     * @param object Object on which access is tested
     * @param subject Subject who seeks to perform the action
     * @param action Action which is to be performed
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    public void checkPermission(ObjectNode object, SubjectNode subject,
                                ActionNode action)
        throws ServiceAccessException, AccessDeniedException,
        ObjectNotFoundException {
        
        if (!hasPermission(object, subject, action)) {
            throw new AccessDeniedException(object.getUri(), subject.getUri(),
                                            action.getUri());
        }
        
    }
    
    
    /**
     * Check whether or not an actor can perform the specified activity
     * on a collection.
     *
     * @param object Object on which access is tested
     * @param subject Subject who seeks to perform the action
     * @param action Action which is to be performed
     * @return true if the action can be performed
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     */
    public boolean hasPermission(ObjectNode object, SubjectNode subject,
                                 ActionNode action)
        throws ServiceAccessException, ObjectNotFoundException {
        
        boolean granted = false;
        boolean denied = false;
        boolean rootObjectReached = false;
        
        ObjectNode courObject = object;
        
        Uri subjectUri = namespace.getUri(subject.getUri());
        Uri actionUri = namespace.getUri(action.getUri());
        
        while (!granted && !denied && !rootObjectReached) {
            
            Uri courUri = namespace.getUri(courObject.getUri());
            Enumeration permissions = courUri.getStore()
                .enumeratePermissions(courUri);
            
            while (permissions.hasMoreElements()) {
                
                boolean oldGranted = granted;
                boolean oldDenied = denied;
                
                NodePermission permission =
                    (NodePermission) permissions.nextElement();
                String permissionSubject = permission.getSubjectUri();
                
                if (permissionSubject.equals("~")) {
                    
                    boolean check;
                    if (permission.isInheritable()) {
                        check =
                            object.getUri().startsWith(subjectUri.toString());
                    } else {
                        check = object.getUri().equals(subjectUri.toString());
                    }
                    
                    // Self permission
                    granted = (!permission.isNegative())
                        && (check)
                        && (actionUri.toString()
                            .startsWith(permission.getActionUri()));
                    denied = (permission.isNegative())
                        && (check)
                        && (actionUri.toString()
                            .startsWith(permission.getActionUri()));
                    
                } else if (permission.isInheritable()
                    || permission.getObjectUri().equals(object.getUri())) {
                    
                    if (permissionSubject.startsWith("/")) {
                        
                        // Node permission
                        granted = (!permission.isNegative())
                            && (subjectUri.toString()
                                .startsWith(permission.getSubjectUri()))
                            && (actionUri.toString()
                                .startsWith(permission.getActionUri()));
                        denied = (permission.isNegative())
                            && (subjectUri.toString()
                                .startsWith(permission.getSubjectUri()))
                            && (actionUri.toString()
                                .startsWith(permission.getActionUri()));
                        
                    } else if (permissionSubject.startsWith("+")) {
                        
                        // Permission group which needs to be expanded
                        Uri permissionSubjectUri =
                            namespace.getUri(permissionSubject.substring(1));
                        ObjectNode group =
                            permissionSubjectUri.getStore().retrieveObject
                            (permissionSubjectUri);
                        // if the node is a GroupNode, expand it out to
                        // normal permissions
                        if (group instanceof
                            org.apache.slide.structure.GroupNode ) {
                            if (group.hasChildren()) {
                                Enumeration groupMembers =
                                    group.enumerateChildren();
                                // parse thru the children of the group and
                                // check permissions on each
                                while (groupMembers.hasMoreElements()) {
                                    
                                    oldGranted = granted;
                                    oldDenied = denied;
                                    
                                    Uri childUri =
                                        namespace.getUri
                                        ((String) groupMembers.nextElement());
                                    ObjectNode childNode =
                                        childUri.getStore().retrieveObject
                                        (childUri);
                                    String childSubjectUri = childNode
                                        instanceof LinkNode ?
                                        ((LinkNode) childNode)
                                        .getLinkedUri() :
                                        childNode.getUri() ;
                                    
                                    granted = (!permission.isNegative())
                                        && (subjectUri.toString()
                                            .startsWith(childSubjectUri))
                                        && (actionUri.toString()
                                            .startsWith
                                            (permission.getActionUri()));
                                    denied = (permission.isNegative())
                                        && (subjectUri.toString()
                                            .startsWith(childSubjectUri))
                                        && (actionUri.toString()
                                            .startsWith
                                            (permission.getActionUri()));
                                    
                                    granted = granted | oldGranted;
                                    denied = denied | oldDenied;
                                    
                                }
                            }
                        }
                        
                    } else {
                        
                        // Role permission
                        granted = (!permission.isNegative())
                            && (hasRole(subject, permissionSubject))
                            && (actionUri.toString()
                                .startsWith(permission.getActionUri()));
                        denied = (permission.isNegative())
                            && (hasRole(subject, permissionSubject))
                            && (actionUri.toString()
                                .startsWith(permission.getActionUri()));
                        
                    }
                    
                }
                
                granted = granted | oldGranted;
                denied = denied | oldDenied;
                
            }
            
            Uri parentUri = courUri.getParentUri();
            
            if (parentUri != null) {
                courObject = parentUri.getStore()
                    .retrieveObject(parentUri);
            } else {
                rootObjectReached = true;
            }
        }
        
        // Negative permissions have priority (if they're defined on the same
        // node)
        if (denied) {
            return false;
        }
        
        if (!granted) {
            return false;
        }
        
        return true;
        
    }
    
    
    /**
     * Enumerates permissions on an object.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    public Enumeration enumeratePermissions(SlideToken token,
                                            ObjectNode object)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException {
        return enumeratePermissions(token, object.getUri());
    }
    
    
    /**
     * Enumerates permissions on an object.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    public Enumeration enumeratePermissions(SlideToken token,
                                            String object)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException {
        Uri objectUri = namespace.getUri(token, object);
        return objectUri.getStore().enumeratePermissions(objectUri);
    }
    
    
    /**
     * Check whether or not the current user has the specified role.
     *
     * @param token Credentials token
     * @param role Role
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     */
    public boolean hasRole(SlideToken token, String role)
        throws ServiceAccessException, ObjectNotFoundException {
        
        ObjectNode subject = getPrincipal(token);
        
        return hasRole(subject, role);
        
    }
    
    
    /**
     * Check whether or not the current user has the specified role.
     *
     * @param object Object node
     * @param role Role
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     */
    public boolean hasRole(ObjectNode object, String role)
        throws ServiceAccessException, ObjectNotFoundException {
        
        if (role.equals(namespaceConfig.NOBODY))
            return true;
        String associatedRole = namespaceConfig.getRoleMapping(role);
        if ((associatedRole != null)
            && (associatedRole.equals(namespaceConfig.NOBODY)))
            return true;
        
        Class roleClass = (Class) rolesCache.get(role);
        if ((roleClass == null) && (associatedRole != null)) {
            roleClass = (Class) rolesCache.get(associatedRole);
            if (roleClass == null) {
                try {
                    roleClass = Class.forName(associatedRole);
                    rolesCache.put(role, roleClass);
                    rolesCache.put(associatedRole, roleClass);
                } catch (ClassNotFoundException ex) {
                }
            }
        }
        if (roleClass == null) {
            try {
                roleClass = Class.forName(role);
                rolesCache.put(role, roleClass);
            } catch (ClassNotFoundException e) {
            }
        }
        if (roleClass == null) {
            return false;
        }
        
        if (roleClass.isInstance(object))
            return true;
        
        return false;
        
    }
    
    
    /**
     * Return the list of roles the specified node has.
     *
     * @param object Object node
     */
    public Enumeration getRoles(ObjectNode object) {
        
        Vector result = new Vector();
        result.addElement(namespaceConfig.NOBODY);
        
        Class currentObjectClass = object.getClass();
        
        while (!currentObjectClass.equals(ObjectNode.class)) {
            
            Class[] interfaces = currentObjectClass.getInterfaces();
            for (int i = 0; i < interfaces.length; i++) {
                String className = interfaces[i].getName();
                String associatedName =
                    namespaceConfig.getRoleMapping(className);
                if (associatedName != null)
                    result.addElement(associatedName);
                else
                    result.addElement(className);
            }
            
            currentObjectClass = currentObjectClass.getSuperclass();
            if (currentObjectClass == null) {
                // Illegal state
                throw new IllegalStateException("Invalid node");
            }
            
        }
        
        return result.elements();
        
    }
    
    
    /**
     * Return the list of roles the specified token has.
     *
     * @param token Credentials token
     */
    public Enumeration getRoles(SlideToken token)
        throws ServiceAccessException, ObjectNotFoundException {
        return getRoles(getPrincipal(token));
    }
    
    
    /**
     * Get the principal associated with the credentials token.
     *
     * @param token Creadentials token
     */
    public ObjectNode getPrincipal(SlideToken token)
        throws ServiceAccessException, ObjectNotFoundException {
        
        String principalPath =
            token.getCredentialsToken().getPublicCredentials();
        if ((principalPath == null) || (principalPath.equals(""))) {
            principalPath = namespaceConfig.getGuestPath();
        }
        
        Uri subjectUri = namespace.getUri
            (token, namespaceConfig.getUsersPath() + "/" + principalPath);
        
        try {
            return subjectUri.getStore().retrieveObject(subjectUri);
        } catch (ObjectNotFoundException e) {
            if (!namespaceConfig.isAutoCreateUsers()) {
                throw e;
            } else {
                try {
                    
                    Uri parentUri = subjectUri.getParentUri();
                    ObjectNode parent =
                        subjectUri.getStore().retrieveObject(parentUri);
                    Enumeration childrenEnum = parent.enumerateChildren();
                    Enumeration linksEnum = parent.enumerateLinks();
                    Vector children = new Vector();
                    while (childrenEnum.hasMoreElements()) {
                        children.addElement(childrenEnum.nextElement());
                    }
                    children.addElement(subjectUri.toString());
                    Vector links = new Vector();
                    while (linksEnum.hasMoreElements()) {
                        links.addElement(linksEnum.nextElement());
                    }
                    
                    // First, load the object's class
                    Class objectClass = Class.forName
                        (namespaceConfig.getAutoCreateUsersRole());
                    Class[] types = { String.class };
                    Object[] args = { subjectUri.toString() };
                    Constructor constructor =
                        objectClass.getConstructor(types);
                    ObjectNode object =
                        (ObjectNode) constructor.newInstance(args);
                    subjectUri.getStore().createObject(subjectUri, object);
                    
                    Class[] types2 =
                    { String.class, Vector.class, Vector.class };
                    Object[] args2 = { parentUri.toString(), children, links };
                    constructor = parent.getClass().getConstructor(types2);
                    object = (ObjectNode) constructor.newInstance(args2);
                    parentUri.getStore().storeObject(parentUri, object);
                } catch (ClassNotFoundException ex) {
                    // Can't find role implementing class
                    throw new ObjectNotFoundException(subjectUri);
                } catch (NoSuchMethodException ex) {
                    // Can't find appropriate constructor
                    throw new ObjectNotFoundException(subjectUri);
                } catch (InstantiationException ex) {
                    // Can't instatiate object
                    throw new ObjectNotFoundException(subjectUri);
                } catch (InvocationTargetException ex) {
                    // Can't invoke constructor
                    throw new ObjectNotFoundException(subjectUri);
                } catch (IllegalAccessException ex) {
                    // Constructor is not public
                    throw new ObjectNotFoundException(subjectUri);
                } catch (ObjectAlreadyExistsException ex) {
                    // Should never happen
                    e.printStackTrace();
                    throw new ObjectNotFoundException(subjectUri);
                }
                return subjectUri.getStore().retrieveObject(subjectUri);
            }
        }
    }


    
}
