/*
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */


package org.apache.slide.webdav.logger;

import java.io.*;
import org.apache.slide.common.Domain;

/**
 * Un-synchronized byte buffer. We have methods to write and read from the
 * buffer, and helpers can convert various data formats.
 *
 * The idea is to minimize the number of buffers and the amount of copy from
 * layer to layer. It's _not_ premature optimization - it's the way things
 * should work.
 *
 * The Request and Response will use several buffers, same for the protocol
 * adapters.
 *
 * Note that the Buffer owns his byte[], while the Chunk is just a light
 * cursor.
 *
 *
 * @author Costin Manolache
 */


public class XByteBuffer {
    // everything happens inside one thread !!!
        
    protected static final int DEFAULT_BUFFER_SIZE = 8*1024;
    int defaultBufferSize = DEFAULT_BUFFER_SIZE;
    int bytesWritten = 0;

    /** The buffers
     */
    private byte buf[];
    private byte buf2[];

    /**
     * The index one greater than the index of the last valid byte in
     * the buffer.
     */
    public int count = 0;
    // count==-1 for end of stream
    

    public XByteBuffer() {
        buf = new byte[defaultBufferSize];
    }

    public void write(int b) throws IOException {
        if (count >= buf.length) {
            resize();
        }
        buf[count++] = (byte)b;
        bytesWritten++;
    }

    public void write(byte b[], int off, int len) throws IOException {
        int avail = buf.length - count;
        Domain.debug("ENTER: XByteBuffer:write(len="+len+") avail="+avail);

        if ( len > avail ) {
            int newLength = buf.length;
            do {
                newLength = newLength + newLength / 2;
            } while ( newLength <= count + len );
            resize( newLength );
        }

        if ( len > 0 ) {
            System.arraycopy(b, off, buf, count, len);
            count += len;
            bytesWritten += len;
        }
        Domain.debug("LEAVE: XByteBuffer:write(len="+len+") bytesWritten="+bytesWritten);
        return;
    }

    /**
     * enlarge the buffer.
     * @pre   buf != null
     */
    private void resize() {
        resize( buf.length + buf.length / 2 );
    }

    /**
     * enlarge the buffer.
     * @pre   buf != null
     */
    private void resize( int neededSize ) {
        Domain.debug("XByteBuffer:resize( neededSize="+neededSize+")");
        buf2 = new byte[neededSize];
        System.arraycopy( buf, 0, buf2, 0, buf.length );
        buf = buf2;
        buf2 = null;
    }

    public boolean isContentWritten() {
    return bytesWritten!=0;
    }
    
    public void setBufferSize(int size) {
    if( size > buf.length ) {
        buf=new byte[size];
    }
    }

    public int getBufferSize() {
    return buf.length;
    }


    // -------------------- Utils

    public byte[] getBuffer() {
        return buf;
    }

    public String getBufferContent() {
        return new String( buf ).substring( 0, bytesWritten );
//hak       return new String( buf );
    }

    public int getNumberOfBytesWritten() {
        return bytesWritten;
    }
    
}
